#include <sys/types.h>

#include <errno.h>
#include <unistd.h>

#include "common.h"


/* 
 * Return number of bytes copied into buffer (excluding terminating nul byte),
 * or 0 on EOF (no bytes read), or -1 on error.  On error, errno is set.
 *
 * If a newline is read, it is returned in the buffer.
 */
ssize_t
read_line(int fd, void *buffer, size_t n)
{
    ssize_t num_read;
    size_t tot_read;
    char *buf;
    char ch;

    if (n <= 0 || buffer == NULL) {
        errno = -EINVAL;
        return -1;
    }

    buf = buffer;                   /* no pointer arithmetic on 'void *' */

    tot_read = 0;
    while (1) {
        num_read = read(fd, &ch, 1);
        if (num_read == -1) {
            if (errno == EINTR)
                continue;
            else
                return -1;
        } else if (num_read == 0) { /* EOF */
            if (tot_read == 0)      /* No bytes read; return 0 */
                return 0;
            else                    /* Some bytes read; add '\0' */
                break;
        } else {                    /* num_read must be 1 if we get here */
            if (tot_read < (n - 1))  {  /* discard > (n-1) bytes */
                tot_read++;
                *buf++ = ch;
            }

            if (ch == '\n')
                break;
        }
    }

    *buf = '\0';
    return tot_read;
}
