#include <sys/types.h>
#include <sys/socket.h>

#include <arpa/inet.h>

#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "common.h"
#include "mu.h"


static void
repl(int sk)
{
    ssize_t len_ret = 0;
    size_t len = 0;
    char * line = NULL;
    char buffer[BUF_SIZE] = { 0 };
    int err;

    while (1) {
        printf("> ");
        len_ret = getline(&line, &len, stdin);
        if (len_ret == -1)
            break;

        err = mu_write_n(sk, line, len_ret, NULL);
        if (err != 0) {
            mu_stderr_errno(-err, "error writing to socket");
            break;
        }

        len_ret = read_line(sk, buffer, sizeof(buffer));
        if (len_ret == 0) {
            printf("server closed connection\n");
            break;
        }

        if (len_ret == -1) {
            mu_stderr_errno(errno, "error reading from socket");
            break;
        }

        printf("%s", buffer);
    }
}


int
main(int argc, char *argv[])
{
    int sk;
    struct sockaddr_in sa;
    int err;

    if (argc != 3)
        mu_die("Usage: %s IP PORT", argv[0]);

    sk = socket(AF_INET, SOCK_STREAM, 0);
    if (sk == -1)
        mu_die_errno(errno, "socket");

    mu_init_sockaddr_in(&sa, argv[1], argv[2]);
    err = connect(sk, (struct sockaddr *)&sa, sizeof(sa));
    if (err == -1)
        mu_die_errno(errno, "connect");

    repl(sk);

    close(sk);

    return 0;
}
