#include <sys/types.h>
#include <sys/socket.h>

#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdint.h>
#include <unistd.h>

#include "common.h"
#include "mu.h"


#define BACKLOG 5


static void
handle_client(int c)
{
    char buffer[BUF_SIZE] = { 0 };
    ssize_t n;
    int i;
    int err;

    while (1) {
        n = read_line(c, buffer, sizeof(buffer));
        if (n == 0) {
            printf("client closed connection\n");
            break;
        }

        if (n == -1) {
            mu_stderr_errno(errno, "error reading from socket");
            break;
        }

        for (i = 0; i < n; i++)
            buffer[i] = toupper(buffer[i]);

        err = mu_write_n(c, buffer, n, NULL);
        if (err != 0)
            mu_stderr_errno(-err, "error writing to socket");
    }

    return;
}


static void
serve_forever(int sk)
{
    int c;
    struct sockaddr_in peer;
    socklen_t addrlen;
    char peer_ip[INET_ADDRSTRLEN] = { 0 };
    uint16_t peer_port;

    while (1) {
        addrlen = sizeof(peer);
        c = accept(sk, (struct sockaddr *)&peer, &addrlen);
        if (c == -1)
            mu_die_errno(errno, "accept");

        if (inet_ntop(AF_INET, &peer.sin_addr, peer_ip, sizeof(peer_ip)) == NULL)
            mu_die_errno(errno, "inet_ntop");
        peer_port = ntohs(peer.sin_port);

        printf("connection from %s:%u\n", peer_ip, peer_port);

        handle_client(c);

        printf("closing our connection\n");
        close(c);
    }
}


static int
tcp_server_create(const char *ip, const char *port)
{
    struct sockaddr_in sa;
    int sk;
    int err;

    sk = socket(AF_INET, SOCK_STREAM, 0);
    if (sk == -1)
        mu_die_errno(errno, "socket");

    mu_reuseaddr(sk);

    mu_init_sockaddr_in(&sa, ip, port);
    err = bind(sk, (struct sockaddr *)&sa, sizeof(sa));
    if (err == -1)
        mu_die_errno(errno, "bind");

    err = listen(sk, BACKLOG);
    if (err == -1)
        mu_die_errno(errno, "listen");

    return sk;
}


int
main(int argc, char *argv[])
{
    int sk;

    if (argc != 3)
        mu_die("Usage: %s IP PORT", argv[0]);

    sk = tcp_server_create(argv[1], argv[2]);
    serve_forever(sk);

    return 0;
}
